@@ -22,24 +22,25 @@ from typing import TYPE_CHECKING
 
 import yaml
 from openlineage.client import OpenLineageClient, set_producer
-from openlineage.client.facet import (
-    BaseFacet,
-    DocumentationJobFacet,
-    ErrorMessageRunFacet,
-    JobTypeJobFacet,
-    NominalTimeRunFacet,
-    OwnershipJobFacet,
-    OwnershipJobFacetOwners,
-    ParentRunFacet,
-    ProcessingEngineRunFacet,
-    SourceCodeLocationJobFacet,
+from openlineage.client.event_v2 import Job, Run, RunEvent, RunState
+from openlineage.client.facet_v2 import (
+    JobFacet,
+    RunFacet,
+    documentation_job,
+    error_message_run,
+    job_type_job,
+    nominal_time_run,
+    ownership_job,
+    parent_run,
+    processing_engine_run,
+    source_code_location_job,
 )
-from openlineage.client.run import Job, Run, RunEvent, RunState
 from openlineage.client.uuid import generate_static_uuid
 
 from airflow.providers.openlineage import __version__ as OPENLINEAGE_PROVIDER_VERSION, conf
 from airflow.providers.openlineage.utils.utils import (
     OpenLineageRedactor,
+    get_airflow_dag_run_facet,
     get_airflow_state_run_facet,
 )
 from airflow.stats import Stats
@@ -59,8 +60,8 @@ set_producer(_PRODUCER)
 # https://openlineage.io/docs/spec/facets/job-facets/job-type
 # They must be set after the `set_producer(_PRODUCER)`
 # otherwise the `JobTypeJobFacet._producer` will be set with the default value
-_JOB_TYPE_DAG = JobTypeJobFacet(jobType="DAG", integration="AIRFLOW", processingType="BATCH")
-_JOB_TYPE_TASK = JobTypeJobFacet(jobType="TASK", integration="AIRFLOW", processingType="BATCH")
+_JOB_TYPE_DAG = job_type_job.JobTypeJobFacet(jobType="DAG", integration="AIRFLOW", processingType="BATCH")
+_JOB_TYPE_TASK = job_type_job.JobTypeJobFacet(jobType="TASK", integration="AIRFLOW", processingType="BATCH")
 
 
 class OpenLineageAdapter(LoggingMixin):
@@ -148,7 +149,7 @@ class OpenLineageAdapter(LoggingMixin):
         if not self._client:
             self._client = self.get_or_create_openlineage_client()
         redacted_event: RunEvent = self._redacter.redact(event, max_depth=20)  # type: ignore[assignment]
-        event_type = event.eventType.value.lower()
+        event_type = event.eventType.value.lower() if event.eventType else ""
         transport_type = f"{self._client.transport.kind}".lower()
 
         try:
@@ -177,7 +178,7 @@ class OpenLineageAdapter(LoggingMixin):
         nominal_end_time: str | None,
         owners: list[str],
         task: OperatorLineage | None,
-        run_facets: dict[str, BaseFacet] | None = None,  # Custom run facets
+        run_facets: dict[str, RunFacet] | None = None,
     ) -> RunEvent:
         """
         Emit openlineage event of type START.
@@ -198,14 +199,13 @@ class OpenLineageAdapter(LoggingMixin):
         """
         from airflow.version import version as AIRFLOW_VERSION
 
-        processing_engine_version_facet = ProcessingEngineRunFacet(
+        processing_engine_version_facet = processing_engine_run.ProcessingEngineRunFacet(
             version=AIRFLOW_VERSION,
             name="Airflow",
             openlineageAdapterVersion=OPENLINEAGE_PROVIDER_VERSION,
         )
 
-        if not run_facets:
-            run_facets = {}
+        run_facets = run_facets or {}
         if task:
             run_facets = {**task.run_facets, **run_facets}
         run_facets["processing_engine"] = processing_engine_version_facet  # type: ignore
@@ -243,6 +243,7 @@ class OpenLineageAdapter(LoggingMixin):
         parent_run_id: str | None,
         end_time: str,
         task: OperatorLineage,
+        run_facets: dict[str, RunFacet] | None = None,
     ) -> RunEvent:
         """
         Emit openlineage event of type COMPLETE.
@@ -254,7 +255,11 @@ class OpenLineageAdapter(LoggingMixin):
         :param parent_run_id: identifier of job spawning this task
         :param end_time: time of task completion
         :param task: metadata container with information extracted from operator
+        :param run_facets: additional run facets
         """
+        run_facets = run_facets or {}
+        if task:
+            run_facets = {**task.run_facets, **run_facets}
         event = RunEvent(
             eventType=RunState.COMPLETE,
             eventTime=end_time,
@@ -263,7 +268,7 @@ class OpenLineageAdapter(LoggingMixin):
                 job_name=job_name,
                 parent_job_name=parent_job_name,
                 parent_run_id=parent_run_id,
-                run_facets=task.run_facets,
+                run_facets=run_facets,
             ),
             job=self._build_job(job_name, job_type=_JOB_TYPE_TASK, job_facets=task.job_facets),
             inputs=task.inputs,
@@ -281,6 +286,7 @@ class OpenLineageAdapter(LoggingMixin):
         end_time: str,
         task: OperatorLineage,
         error: str | BaseException | None = None,
+        run_facets: dict[str, RunFacet] | None = None,
     ) -> RunEvent:
         """
         Emit openlineage event of type FAIL.
@@ -292,21 +298,23 @@ class OpenLineageAdapter(LoggingMixin):
         :param parent_run_id: identifier of job spawning this task
         :param end_time: time of task completion
         :param task: metadata container with information extracted from operator
+        :param run_facets: custom run facets
         :param error: error
+        :param run_facets: additional run facets
         """
-        error_facet = {}
+        run_facets = run_facets or {}
+        if task:
+            run_facets = {**task.run_facets, **run_facets}
+
         if error:
             stack_trace = None
             if isinstance(error, BaseException) and error.__traceback__:
                 import traceback
 
                 stack_trace = "\\n".join(traceback.format_exception(type(error), error, error.__traceback__))
-
-            error_facet = {
-                "errorMessage": ErrorMessageRunFacet(
-                    message=str(error), programmingLanguage="python", stackTrace=stack_trace
-                )
-            }
+            run_facets["errorMessage"] = error_message_run.ErrorMessageRunFacet(
+                message=str(error), programmingLanguage="python", stackTrace=stack_trace
+            )
 
         event = RunEvent(
             eventType=RunState.FAIL,
@@ -316,7 +324,7 @@ class OpenLineageAdapter(LoggingMixin):
                 job_name=job_name,
                 parent_job_name=parent_job_name,
                 parent_run_id=parent_run_id,
-                run_facets={**task.run_facets, **error_facet},
+                run_facets=run_facets,
             ),
             job=self._build_job(job_name, job_type=_JOB_TYPE_TASK, job_facets=task.job_facets),
             inputs=task.inputs,
@@ -331,9 +339,10 @@ class OpenLineageAdapter(LoggingMixin):
         msg: str,
         nominal_start_time: str,
         nominal_end_time: str,
-        job_facets: dict[str, BaseFacet] | None = None,  # Custom job facets
+        job_facets: dict[str, JobFacet] | None = None,  # Custom job facets
     ):
         try:
+            owner = [x.strip() for x in dag_run.dag.owner.split(",")] if dag_run.dag else None
             event = RunEvent(
                 eventType=RunState.START,
                 eventTime=dag_run.start_date.isoformat(),
@@ -341,7 +350,7 @@ class OpenLineageAdapter(LoggingMixin):
                     job_name=dag_run.dag_id,
                     job_type=_JOB_TYPE_DAG,
                     job_description=dag_run.dag.description if dag_run.dag else None,
-                    owners=[x.strip() for x in dag_run.dag.owner.split(",")] if dag_run.dag else None,
+                    owners=owner,
                     job_facets=job_facets,
                 ),
                 run=self._build_run(
@@ -352,6 +361,7 @@ class OpenLineageAdapter(LoggingMixin):
                     job_name=dag_run.dag_id,
                     nominal_start_time=nominal_start_time,
                     nominal_end_time=nominal_end_time,
+                    run_facets=get_airflow_dag_run_facet(dag_run),
                 ),
                 inputs=[],
                 outputs=[],
@@ -400,7 +410,9 @@ class OpenLineageAdapter(LoggingMixin):
                         execution_date=dag_run.execution_date,
                     ),
                     facets={
-                        "errorMessage": ErrorMessageRunFacet(message=msg, programmingLanguage="python"),
+                        "errorMessage": error_message_run.ErrorMessageRunFacet(
+                            message=msg, programmingLanguage="python"
+                        ),
                         **get_airflow_state_run_facet(dag_run),
                     },
                 ),
@@ -423,23 +435,19 @@ class OpenLineageAdapter(LoggingMixin):
         parent_run_id: str | None = None,
         nominal_start_time: str | None = None,
         nominal_end_time: str | None = None,
-        run_facets: dict[str, BaseFacet] | None = None,
+        run_facets: dict[str, RunFacet] | None = None,
     ) -> Run:
-        facets: dict[str, BaseFacet] = {}
+        facets: dict[str, RunFacet] = {}
         if nominal_start_time:
-            facets.update({"nominalTime": NominalTimeRunFacet(nominal_start_time, nominal_end_time)})
-        if parent_run_id:
-            parent_run_facet = ParentRunFacet.create(
-                runId=parent_run_id,
-                namespace=conf.namespace(),
-                name=parent_job_name or job_name,
-            )
             facets.update(
-                {
-                    "parent": parent_run_facet,
-                    "parentRun": parent_run_facet,  # Keep sending this for the backward compatibility
-                }
+                {"nominalTime": nominal_time_run.NominalTimeRunFacet(nominal_start_time, nominal_end_time)}
             )
+        if parent_run_id:
+            parent_run_facet = parent_run.ParentRunFacet(
+                run=parent_run.Run(runId=parent_run_id),
+                job=parent_run.Job(namespace=conf.namespace(), name=parent_job_name or job_name),
+            )
+            facets.update({"parent": parent_run_facet})
 
         if run_facets:
             facets.update(run_facets)
@@ -449,23 +457,31 @@ class OpenLineageAdapter(LoggingMixin):
     @staticmethod
     def _build_job(
         job_name: str,
-        job_type: JobTypeJobFacet,
+        job_type: job_type_job.JobTypeJobFacet,
         job_description: str | None = None,
         code_location: str | None = None,
         owners: list[str] | None = None,
-        job_facets: dict[str, BaseFacet] | None = None,
+        job_facets: dict[str, JobFacet] | None = None,
     ):
-        facets: dict[str, BaseFacet] = {}
+        facets: dict[str, JobFacet] = {}
 
         if job_description:
-            facets.update({"documentation": DocumentationJobFacet(description=job_description)})
+            facets.update(
+                {"documentation": documentation_job.DocumentationJobFacet(description=job_description)}
+            )
         if code_location:
-            facets.update({"sourceCodeLocation": SourceCodeLocationJobFacet("", url=code_location)})
+            facets.update(
+                {
+                    "sourceCodeLocation": source_code_location_job.SourceCodeLocationJobFacet(
+                        "", url=code_location
+                    )
+                }
+            )
         if owners:
             facets.update(
                 {
-                    "ownership": OwnershipJobFacet(
-                        owners=[OwnershipJobFacetOwners(name=owner) for owner in owners]
+                    "ownership": ownership_job.OwnershipJobFacet(
+                        owners=[ownership_job.Owner(name=owner) for owner in owners]
                     )
                 }
             )
